pub struct GroundStation {
pub name: String,
pub location: Location,
pub measurement_types: IndexSet<MeasurementType>,
pub integration_time: Option<Duration>,
pub light_time_correction: bool,
pub timestamp_noise_s: Option<StochasticNoise>,
pub stochastic_noises: Option<IndexMap<MeasurementType, StochasticNoise>>,
}Expand description
GroundStation defines a one-way or two-way ranging and doppler station. Set the integration time for two-way.
Fields
name: Stringlocation: Locationmeasurement_types: IndexSet<MeasurementType>integration_time: Option<Duration>Duration needed to generate a measurement (if unset, it is assumed to be instantaneous)
light_time_correction: boolWhether to correct for light travel time
timestamp_noise_s: Option<StochasticNoise>Noise on the timestamp of the measurement
stochastic_noises: Option<IndexMap<MeasurementType, StochasticNoise>>Implementations
Sourceimpl GroundStation
impl GroundStation
pub fn dss65_madrid( elevation_mask_deg: f64, range_noise_km: StochasticNoise, doppler_noise_km_s: StochasticNoise, ) -> Self
pub fn dss34_canberra( elevation_mask_deg: f64, range_noise_km: StochasticNoise, doppler_noise_km_s: StochasticNoise, ) -> Self
pub fn dss13_goldstone( elevation_mask_deg: f64, range_noise_km: StochasticNoise, doppler_noise_km_s: StochasticNoise, ) -> Self
Sourceimpl GroundStation
impl GroundStation
Sourcepub fn azimuth_elevation_of(
&self,
rx: Orbit,
obstructing_body: Option<Frame>,
almanac: &Almanac,
) -> AlmanacResult<AzElRange>
pub fn azimuth_elevation_of( &self, rx: Orbit, obstructing_body: Option<Frame>, almanac: &Almanac, ) -> AlmanacResult<AzElRange>
Computes the azimuth and elevation of the provided object seen from this ground station, both in degrees. This is a shortcut to almanac.azimuth_elevation_range_sez.
Sourcepub fn to_orbit(&self, epoch: Epoch, almanac: &Almanac) -> AlmanacResult<Orbit>
pub fn to_orbit(&self, epoch: Epoch, almanac: &Almanac) -> AlmanacResult<Orbit>
Return this ground station as an orbit in its current frame
Examples found in repository?
30fn main() -> Result<(), Box<dyn Error>> {
31 pel::init();
32 // Dynamics models require planetary constants and ephemerides to be defined.
33 // Let's start by grabbing those by using ANISE's latest MetaAlmanac.
34 // This will automatically download the DE440s planetary ephemeris,
35 // the daily-updated Earth Orientation Parameters, the high fidelity Moon orientation
36 // parameters (for the Moon Mean Earth and Moon Principal Axes frames), and the PCK11
37 // planetary constants kernels.
38 // For details, refer to https://github.com/nyx-space/anise/blob/master/data/latest.dhall.
39 // Note that we place the Almanac into an Arc so we can clone it cheaply and provide read-only
40 // references to many functions.
41 let almanac = Arc::new(MetaAlmanac::latest().map_err(Box::new)?);
42 // Define the orbit epoch
43 let epoch = Epoch::from_gregorian_utc_hms(2024, 2, 29, 12, 13, 14);
44
45 // Define the orbit.
46 // First we need to fetch the Earth J2000 from information from the Almanac.
47 // This allows the frame to include the gravitational parameters and the shape of the Earth,
48 // defined as a tri-axial ellipoid. Note that this shape can be changed manually or in the Almanac
49 // by loading a different set of planetary constants.
50 let earth_j2000 = almanac.frame_info(EARTH_J2000)?;
51
52 let orbit =
53 Orbit::try_keplerian_altitude(300.0, 0.015, 68.5, 65.2, 75.0, 0.0, epoch, earth_j2000)?;
54 // Print in in Keplerian form.
55 println!("{orbit:x}");
56
57 // There are two ways to propagate an orbit. We can make a quick approximation assuming only two-body
58 // motion. This is a useful first order approximation but it isn't used in real-world applications.
59
60 // This approach is a feature of ANISE.
61 let future_orbit_tb = orbit.at_epoch(epoch + Unit::Day * 3)?;
62 println!("{future_orbit_tb:x}");
63
64 // Two body propagation relies solely on Kepler's laws, so only the true anomaly will change.
65 println!(
66 "SMA changed by {:.3e} km",
67 orbit.sma_km()? - future_orbit_tb.sma_km()?
68 );
69 println!(
70 "ECC changed by {:.3e}",
71 orbit.ecc()? - future_orbit_tb.ecc()?
72 );
73 println!(
74 "INC changed by {:.3e} deg",
75 orbit.inc_deg()? - future_orbit_tb.inc_deg()?
76 );
77 println!(
78 "RAAN changed by {:.3e} deg",
79 orbit.raan_deg()? - future_orbit_tb.raan_deg()?
80 );
81 println!(
82 "AOP changed by {:.3e} deg",
83 orbit.aop_deg()? - future_orbit_tb.aop_deg()?
84 );
85 println!(
86 "TA changed by {:.3} deg",
87 orbit.ta_deg()? - future_orbit_tb.ta_deg()?
88 );
89
90 // Nyx is used for high fidelity propagation, not Keplerian propagation as above.
91 // Nyx only propagates Spacecraft at the moment, which allows it to account for acceleration
92 // models such as solar radiation pressure.
93
94 // Let's build a cubesat sized spacecraft, with an SRP area of 10 cm^2 and a mass of 9.6 kg.
95 let sc = Spacecraft::builder()
96 .orbit(orbit)
97 .mass(Mass::from_dry_mass(9.60))
98 .srp(SRPData {
99 area_m2: 10e-4,
100 coeff_reflectivity: 1.1,
101 })
102 .build();
103 println!("{sc:x}");
104
105 // Set up the spacecraft dynamics.
106
107 // Specify that the orbital dynamics must account for the graviational pull of the Moon and the Sun.
108 // The gravity of the Earth will also be accounted for since the spaceraft in an Earth orbit.
109 let mut orbital_dyn = OrbitalDynamics::point_masses(vec![MOON, SUN]);
110
111 // We want to include the spherical harmonics, so let's download the gravitational data from the Nyx Cloud.
112 // We're using the JGM3 model here, which is the default in GMAT.
113 let mut jgm3_meta = MetaFile {
114 uri: "http://public-data.nyxspace.com/nyx/models/JGM3.cof.gz".to_string(),
115 crc32: Some(0xF446F027), // Specifying the CRC32 avoids redownloading it if it's cached.
116 };
117 // And let's download it if we don't have it yet.
118 jgm3_meta.process(true)?;
119
120 // Build the spherical harmonics.
121 // The harmonics must be computed in the body fixed frame.
122 // We're using the long term prediction of the Earth centered Earth fixed frame, IAU Earth.
123 let harmonics_21x21 = GravityField::new(
124 GravityFieldData::from_cof(
125 &jgm3_meta.uri,
126 21,
127 21,
128 true,
129 almanac.frame_info(IAU_EARTH_FRAME)?,
130 )
131 .unwrap(),
132 );
133
134 // Include the spherical harmonics into the orbital dynamics.
135 orbital_dyn.accel_models.push(harmonics_21x21);
136
137 // We define the solar radiation pressure, using the default solar flux and accounting only
138 // for the eclipsing caused by the Earth.
139 let srp_dyn = SolarPressure::default_flux(EARTH_J2000, &almanac)?;
140
141 // Finalize setting up the dynamics, specifying the force models (orbital_dyn) separately from the
142 // acceleration models (SRP in this case). Use `from_models` to specify multiple accel models.
143 let dynamics = SpacecraftDynamics::from_model(orbital_dyn, srp_dyn);
144
145 println!("{dynamics}");
146
147 // Finally, let's propagate this orbit to the same epoch as above.
148 // The first returned value is the spacecraft state at the final epoch.
149 // The second value is the full trajectory where the step size is variable step used by the propagator.
150 let (future_sc, trajectory) = Propagator::default(dynamics)
151 .with(sc, almanac.clone())
152 .until_epoch_with_traj(future_orbit_tb.epoch)?;
153
154 println!("=== High fidelity propagation ===");
155 println!(
156 "SMA changed by {:.3} km",
157 orbit.sma_km()? - future_sc.orbit.sma_km()?
158 );
159 println!(
160 "ECC changed by {:.6}",
161 orbit.ecc()? - future_sc.orbit.ecc()?
162 );
163 println!(
164 "INC changed by {:.3e} deg",
165 orbit.inc_deg()? - future_sc.orbit.inc_deg()?
166 );
167 println!(
168 "RAAN changed by {:.3} deg",
169 orbit.raan_deg()? - future_sc.orbit.raan_deg()?
170 );
171 println!(
172 "AOP changed by {:.3} deg",
173 orbit.aop_deg()? - future_sc.orbit.aop_deg()?
174 );
175 println!(
176 "TA changed by {:.3} deg",
177 orbit.ta_deg()? - future_sc.orbit.ta_deg()?
178 );
179
180 // We also have access to the full trajectory throughout the propagation.
181 println!("{trajectory}");
182
183 // With the trajectory, let's build a few data products.
184
185 // 1. Export the trajectory as a CCSDS OEM version 2.0 file and as a parquet file, which includes the Keplerian orbital elements.
186
187 trajectory.to_oem_file(
188 "./01_cubesat_hf_prop.oem",
189 "CUBESAT-ID".to_string(),
190 Some("Nyx Space".to_string()),
191 Some("CUBESAT".to_string()),
192 ExportCfg::builder().step(Unit::Minute * 2).build(),
193 )?;
194
195 trajectory.to_parquet_with_cfg(
196 "./01_cubesat_hf_prop.parquet",
197 ExportCfg::builder().step(Unit::Minute * 2).build(),
198 )?;
199
200 // 2. Compare the difference in the radial-intrack-crosstrack frame between the high fidelity
201 // and Keplerian propagation. The RIC frame is commonly used to compute the difference in position
202 // and velocity of different spacecraft.
203 // 3. Compute the azimuth, elevation, range, and range-rate data of that spacecraft as seen from Boulder, CO, USA.
204
205 let boulder_station = GroundStation::from_point(
206 "Boulder, CO, USA".to_string(),
207 40.014984, // latitude in degrees
208 -105.270546, // longitude in degrees
209 1.6550, // altitude in kilometers
210 almanac.frame_info(IAU_EARTH_FRAME)?,
211 );
212
213 // We iterate over the trajectory, grabbing a state every two minutes.
214 let mut offset_s = vec![];
215 let mut epoch_str = vec![];
216 let mut ric_x_km = vec![];
217 let mut ric_y_km = vec![];
218 let mut ric_z_km = vec![];
219 let mut ric_vx_km_s = vec![];
220 let mut ric_vy_km_s = vec![];
221 let mut ric_vz_km_s = vec![];
222
223 let mut azimuth_deg = vec![];
224 let mut elevation_deg = vec![];
225 let mut range_km = vec![];
226 let mut range_rate_km_s = vec![];
227 for state in trajectory.every(Unit::Minute * 2) {
228 // Try to compute the Keplerian/two body state just in time.
229 // This method occasionally fails to converge on an appropriate true anomaly
230 // from the mean anomaly. If that happens, we just skip this state.
231 // The high fidelity and Keplerian states diverge continuously, and we're curious
232 // about the divergence in this quick analysis.
233 let this_epoch = state.epoch();
234 match orbit.at_epoch(this_epoch) {
235 Ok(tb_then) => {
236 offset_s.push((this_epoch - orbit.epoch).to_seconds());
237 epoch_str.push(format!("{this_epoch}"));
238 // Compute the two body state just in time.
239 let ric = state.orbit.ric_difference(&tb_then)?;
240 ric_x_km.push(ric.radius_km.x);
241 ric_y_km.push(ric.radius_km.y);
242 ric_z_km.push(ric.radius_km.z);
243 ric_vx_km_s.push(ric.velocity_km_s.x);
244 ric_vy_km_s.push(ric.velocity_km_s.y);
245 ric_vz_km_s.push(ric.velocity_km_s.z);
246
247 // Compute the AER data for each state.
248 let aer = almanac.azimuth_elevation_range_sez(
249 state.orbit,
250 boulder_station.to_orbit(this_epoch, &almanac)?,
251 None,
252 None,
253 )?;
254 azimuth_deg.push(aer.azimuth_deg);
255 elevation_deg.push(aer.elevation_deg);
256 range_km.push(aer.range_km);
257 range_rate_km_s.push(aer.range_rate_km_s);
258 }
259 Err(e) => warn!("{} {e}", state.epoch()),
260 };
261 }
262
263 // Build the data frames.
264 let ric_df = df!(
265 "Offset (s)" => offset_s.clone(),
266 "Epoch" => epoch_str.clone(),
267 "RIC X (km)" => ric_x_km,
268 "RIC Y (km)" => ric_y_km,
269 "RIC Z (km)" => ric_z_km,
270 "RIC VX (km/s)" => ric_vx_km_s,
271 "RIC VY (km/s)" => ric_vy_km_s,
272 "RIC VZ (km/s)" => ric_vz_km_s,
273 )?;
274
275 println!("RIC difference at start\n{}", ric_df.head(Some(10)));
276 println!("RIC difference at end\n{}", ric_df.tail(Some(10)));
277
278 let aer_df = df!(
279 "Offset (s)" => offset_s.clone(),
280 "Epoch" => epoch_str.clone(),
281 "azimuth (deg)" => azimuth_deg,
282 "elevation (deg)" => elevation_deg,
283 "range (km)" => range_km,
284 "range rate (km/s)" => range_rate_km_s,
285 )?;
286
287 // Finally, let's see when the spacecraft is visible, assuming 15 degrees minimum elevation.
288 let mask = aer_df
289 .column("elevation (deg)")?
290 .gt(&Column::Scalar(ScalarColumn::new(
291 "elevation mask (deg)".into(),
292 Scalar::new(DataType::Float64, AnyValue::Float64(15.0)),
293 offset_s.len(),
294 )))?;
295 let cubesat_visible = aer_df.filter(&mask)?;
296
297 println!("{cubesat_visible}");
298
299 Ok(())
300}Sourceimpl GroundStation
impl GroundStation
Sourcepub fn from_point(
name: String,
latitude_deg: f64,
longitude_deg: f64,
height_km: f64,
frame: Frame,
) -> Self
pub fn from_point( name: String, latitude_deg: f64, longitude_deg: f64, height_km: f64, frame: Frame, ) -> Self
Initializes a point on the surface of a celestial object. This is meant for analysis, not for spacecraft navigation.
Examples found in repository?
30fn main() -> Result<(), Box<dyn Error>> {
31 pel::init();
32 // Dynamics models require planetary constants and ephemerides to be defined.
33 // Let's start by grabbing those by using ANISE's latest MetaAlmanac.
34 // This will automatically download the DE440s planetary ephemeris,
35 // the daily-updated Earth Orientation Parameters, the high fidelity Moon orientation
36 // parameters (for the Moon Mean Earth and Moon Principal Axes frames), and the PCK11
37 // planetary constants kernels.
38 // For details, refer to https://github.com/nyx-space/anise/blob/master/data/latest.dhall.
39 // Note that we place the Almanac into an Arc so we can clone it cheaply and provide read-only
40 // references to many functions.
41 let almanac = Arc::new(MetaAlmanac::latest().map_err(Box::new)?);
42 // Define the orbit epoch
43 let epoch = Epoch::from_gregorian_utc_hms(2024, 2, 29, 12, 13, 14);
44
45 // Define the orbit.
46 // First we need to fetch the Earth J2000 from information from the Almanac.
47 // This allows the frame to include the gravitational parameters and the shape of the Earth,
48 // defined as a tri-axial ellipoid. Note that this shape can be changed manually or in the Almanac
49 // by loading a different set of planetary constants.
50 let earth_j2000 = almanac.frame_info(EARTH_J2000)?;
51
52 let orbit =
53 Orbit::try_keplerian_altitude(300.0, 0.015, 68.5, 65.2, 75.0, 0.0, epoch, earth_j2000)?;
54 // Print in in Keplerian form.
55 println!("{orbit:x}");
56
57 // There are two ways to propagate an orbit. We can make a quick approximation assuming only two-body
58 // motion. This is a useful first order approximation but it isn't used in real-world applications.
59
60 // This approach is a feature of ANISE.
61 let future_orbit_tb = orbit.at_epoch(epoch + Unit::Day * 3)?;
62 println!("{future_orbit_tb:x}");
63
64 // Two body propagation relies solely on Kepler's laws, so only the true anomaly will change.
65 println!(
66 "SMA changed by {:.3e} km",
67 orbit.sma_km()? - future_orbit_tb.sma_km()?
68 );
69 println!(
70 "ECC changed by {:.3e}",
71 orbit.ecc()? - future_orbit_tb.ecc()?
72 );
73 println!(
74 "INC changed by {:.3e} deg",
75 orbit.inc_deg()? - future_orbit_tb.inc_deg()?
76 );
77 println!(
78 "RAAN changed by {:.3e} deg",
79 orbit.raan_deg()? - future_orbit_tb.raan_deg()?
80 );
81 println!(
82 "AOP changed by {:.3e} deg",
83 orbit.aop_deg()? - future_orbit_tb.aop_deg()?
84 );
85 println!(
86 "TA changed by {:.3} deg",
87 orbit.ta_deg()? - future_orbit_tb.ta_deg()?
88 );
89
90 // Nyx is used for high fidelity propagation, not Keplerian propagation as above.
91 // Nyx only propagates Spacecraft at the moment, which allows it to account for acceleration
92 // models such as solar radiation pressure.
93
94 // Let's build a cubesat sized spacecraft, with an SRP area of 10 cm^2 and a mass of 9.6 kg.
95 let sc = Spacecraft::builder()
96 .orbit(orbit)
97 .mass(Mass::from_dry_mass(9.60))
98 .srp(SRPData {
99 area_m2: 10e-4,
100 coeff_reflectivity: 1.1,
101 })
102 .build();
103 println!("{sc:x}");
104
105 // Set up the spacecraft dynamics.
106
107 // Specify that the orbital dynamics must account for the graviational pull of the Moon and the Sun.
108 // The gravity of the Earth will also be accounted for since the spaceraft in an Earth orbit.
109 let mut orbital_dyn = OrbitalDynamics::point_masses(vec![MOON, SUN]);
110
111 // We want to include the spherical harmonics, so let's download the gravitational data from the Nyx Cloud.
112 // We're using the JGM3 model here, which is the default in GMAT.
113 let mut jgm3_meta = MetaFile {
114 uri: "http://public-data.nyxspace.com/nyx/models/JGM3.cof.gz".to_string(),
115 crc32: Some(0xF446F027), // Specifying the CRC32 avoids redownloading it if it's cached.
116 };
117 // And let's download it if we don't have it yet.
118 jgm3_meta.process(true)?;
119
120 // Build the spherical harmonics.
121 // The harmonics must be computed in the body fixed frame.
122 // We're using the long term prediction of the Earth centered Earth fixed frame, IAU Earth.
123 let harmonics_21x21 = GravityField::new(
124 GravityFieldData::from_cof(
125 &jgm3_meta.uri,
126 21,
127 21,
128 true,
129 almanac.frame_info(IAU_EARTH_FRAME)?,
130 )
131 .unwrap(),
132 );
133
134 // Include the spherical harmonics into the orbital dynamics.
135 orbital_dyn.accel_models.push(harmonics_21x21);
136
137 // We define the solar radiation pressure, using the default solar flux and accounting only
138 // for the eclipsing caused by the Earth.
139 let srp_dyn = SolarPressure::default_flux(EARTH_J2000, &almanac)?;
140
141 // Finalize setting up the dynamics, specifying the force models (orbital_dyn) separately from the
142 // acceleration models (SRP in this case). Use `from_models` to specify multiple accel models.
143 let dynamics = SpacecraftDynamics::from_model(orbital_dyn, srp_dyn);
144
145 println!("{dynamics}");
146
147 // Finally, let's propagate this orbit to the same epoch as above.
148 // The first returned value is the spacecraft state at the final epoch.
149 // The second value is the full trajectory where the step size is variable step used by the propagator.
150 let (future_sc, trajectory) = Propagator::default(dynamics)
151 .with(sc, almanac.clone())
152 .until_epoch_with_traj(future_orbit_tb.epoch)?;
153
154 println!("=== High fidelity propagation ===");
155 println!(
156 "SMA changed by {:.3} km",
157 orbit.sma_km()? - future_sc.orbit.sma_km()?
158 );
159 println!(
160 "ECC changed by {:.6}",
161 orbit.ecc()? - future_sc.orbit.ecc()?
162 );
163 println!(
164 "INC changed by {:.3e} deg",
165 orbit.inc_deg()? - future_sc.orbit.inc_deg()?
166 );
167 println!(
168 "RAAN changed by {:.3} deg",
169 orbit.raan_deg()? - future_sc.orbit.raan_deg()?
170 );
171 println!(
172 "AOP changed by {:.3} deg",
173 orbit.aop_deg()? - future_sc.orbit.aop_deg()?
174 );
175 println!(
176 "TA changed by {:.3} deg",
177 orbit.ta_deg()? - future_sc.orbit.ta_deg()?
178 );
179
180 // We also have access to the full trajectory throughout the propagation.
181 println!("{trajectory}");
182
183 // With the trajectory, let's build a few data products.
184
185 // 1. Export the trajectory as a CCSDS OEM version 2.0 file and as a parquet file, which includes the Keplerian orbital elements.
186
187 trajectory.to_oem_file(
188 "./01_cubesat_hf_prop.oem",
189 "CUBESAT-ID".to_string(),
190 Some("Nyx Space".to_string()),
191 Some("CUBESAT".to_string()),
192 ExportCfg::builder().step(Unit::Minute * 2).build(),
193 )?;
194
195 trajectory.to_parquet_with_cfg(
196 "./01_cubesat_hf_prop.parquet",
197 ExportCfg::builder().step(Unit::Minute * 2).build(),
198 )?;
199
200 // 2. Compare the difference in the radial-intrack-crosstrack frame between the high fidelity
201 // and Keplerian propagation. The RIC frame is commonly used to compute the difference in position
202 // and velocity of different spacecraft.
203 // 3. Compute the azimuth, elevation, range, and range-rate data of that spacecraft as seen from Boulder, CO, USA.
204
205 let boulder_station = GroundStation::from_point(
206 "Boulder, CO, USA".to_string(),
207 40.014984, // latitude in degrees
208 -105.270546, // longitude in degrees
209 1.6550, // altitude in kilometers
210 almanac.frame_info(IAU_EARTH_FRAME)?,
211 );
212
213 // We iterate over the trajectory, grabbing a state every two minutes.
214 let mut offset_s = vec![];
215 let mut epoch_str = vec![];
216 let mut ric_x_km = vec![];
217 let mut ric_y_km = vec![];
218 let mut ric_z_km = vec![];
219 let mut ric_vx_km_s = vec![];
220 let mut ric_vy_km_s = vec![];
221 let mut ric_vz_km_s = vec![];
222
223 let mut azimuth_deg = vec![];
224 let mut elevation_deg = vec![];
225 let mut range_km = vec![];
226 let mut range_rate_km_s = vec![];
227 for state in trajectory.every(Unit::Minute * 2) {
228 // Try to compute the Keplerian/two body state just in time.
229 // This method occasionally fails to converge on an appropriate true anomaly
230 // from the mean anomaly. If that happens, we just skip this state.
231 // The high fidelity and Keplerian states diverge continuously, and we're curious
232 // about the divergence in this quick analysis.
233 let this_epoch = state.epoch();
234 match orbit.at_epoch(this_epoch) {
235 Ok(tb_then) => {
236 offset_s.push((this_epoch - orbit.epoch).to_seconds());
237 epoch_str.push(format!("{this_epoch}"));
238 // Compute the two body state just in time.
239 let ric = state.orbit.ric_difference(&tb_then)?;
240 ric_x_km.push(ric.radius_km.x);
241 ric_y_km.push(ric.radius_km.y);
242 ric_z_km.push(ric.radius_km.z);
243 ric_vx_km_s.push(ric.velocity_km_s.x);
244 ric_vy_km_s.push(ric.velocity_km_s.y);
245 ric_vz_km_s.push(ric.velocity_km_s.z);
246
247 // Compute the AER data for each state.
248 let aer = almanac.azimuth_elevation_range_sez(
249 state.orbit,
250 boulder_station.to_orbit(this_epoch, &almanac)?,
251 None,
252 None,
253 )?;
254 azimuth_deg.push(aer.azimuth_deg);
255 elevation_deg.push(aer.elevation_deg);
256 range_km.push(aer.range_km);
257 range_rate_km_s.push(aer.range_rate_km_s);
258 }
259 Err(e) => warn!("{} {e}", state.epoch()),
260 };
261 }
262
263 // Build the data frames.
264 let ric_df = df!(
265 "Offset (s)" => offset_s.clone(),
266 "Epoch" => epoch_str.clone(),
267 "RIC X (km)" => ric_x_km,
268 "RIC Y (km)" => ric_y_km,
269 "RIC Z (km)" => ric_z_km,
270 "RIC VX (km/s)" => ric_vx_km_s,
271 "RIC VY (km/s)" => ric_vy_km_s,
272 "RIC VZ (km/s)" => ric_vz_km_s,
273 )?;
274
275 println!("RIC difference at start\n{}", ric_df.head(Some(10)));
276 println!("RIC difference at end\n{}", ric_df.tail(Some(10)));
277
278 let aer_df = df!(
279 "Offset (s)" => offset_s.clone(),
280 "Epoch" => epoch_str.clone(),
281 "azimuth (deg)" => azimuth_deg,
282 "elevation (deg)" => elevation_deg,
283 "range (km)" => range_km,
284 "range rate (km/s)" => range_rate_km_s,
285 )?;
286
287 // Finally, let's see when the spacecraft is visible, assuming 15 degrees minimum elevation.
288 let mask = aer_df
289 .column("elevation (deg)")?
290 .gt(&Column::Scalar(ScalarColumn::new(
291 "elevation mask (deg)".into(),
292 Scalar::new(DataType::Float64, AnyValue::Float64(15.0)),
293 offset_s.len(),
294 )))?;
295 let cubesat_visible = aer_df.filter(&mask)?;
296
297 println!("{cubesat_visible}");
298
299 Ok(())
300}Sourcepub fn with_msr_type(
self,
msr_type: MeasurementType,
noise: StochasticNoise,
) -> Self
pub fn with_msr_type( self, msr_type: MeasurementType, noise: StochasticNoise, ) -> Self
Returns a copy of this ground station with the new measurement type added (or replaced)
Sourcepub fn without_msr_type(self, msr_type: MeasurementType) -> Self
pub fn without_msr_type(self, msr_type: MeasurementType) -> Self
Returns a copy of this ground station without the provided measurement type (if defined, else no error)
pub fn with_integration_time(self, integration_time: Option<Duration>) -> Self
Sourcepub fn with_msr_bias_constant(
self,
msr_type: MeasurementType,
bias_constant: f64,
) -> Result<Self, ODError>
pub fn with_msr_bias_constant( self, msr_type: MeasurementType, bias_constant: f64, ) -> Result<Self, ODError>
Returns a copy of this ground station with the measurement type noises’ constant bias set to the provided value.